#include <ansi_c.h>
#include <utility.h>
#include <formatio.h>
#include "hp34401.h"
#include "ag33250a.h"
#include <cvirte.h>		
#include <userint.h>
#include "automated.h"

static int panelHandle;
static ViSession instrumentHandle1;
static ViSession instrumentHandle2;
unsigned char process;

unsigned int amplituda,frequency,deltaf;
unsigned int current_point,points;
double measurements[1000];
unsigned int frequencies[1000];
static char file_name[MAX_PATHNAME_LEN]; 
	int file1;   

int main (int argc, char *argv[])
   {
   if (InitCVIRTE (0, argv, 0) == 0)
      return -1;	/* out of memory */
   if ((panelHandle = LoadPanel (0, "automated.uir", PANEL)) < 0)
      return -1;
   DisplayPanel (panelHandle);
	
   hp34401_init ("GPIB0::15", VI_TRUE, VI_TRUE, &instrumentHandle1);
   ag33250a_init ("GPIB::10::INSTR", VI_TRUE, VI_TRUE,
				   &instrumentHandle2);
	
   RunUserInterface ();
	
   ag33250a_close (instrumentHandle2);
	
   hp34401_close (instrumentHandle1);
	
   DiscardPanel (panelHandle);
   return 0;
   }

int CVICALLBACK izlaz (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_COMMIT:
	     QuitUserInterface (0);
	  break;
	  }  
   return 0;
   }

int CVICALLBACK start_generator (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
		  
	  case EVENT_COMMIT:
		 GetCtrlVal (panelHandle, PANEL_AMP_INPUT, &amplituda);
		 GetCtrlVal (panelHandle, PANEL_FREQ_INPUT, &frequency);
	     ag33250a_appl (instrumentHandle2, ag33250a_APPL_SIN, frequency, amplituda, 0.0);
		 break;
      }
   return 0;
   }

int CVICALLBACK multimeter_start (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_COMMIT:
	     hp34401_confVoltAc(instrumentHandle1); //konfigurisanje da se meri AC
		 hp34401_voltAcRangAuto(instrumentHandle1,VI_TRUE); //ukljucenje auto opsega
		 break;
	  }
   return 0;
   }


int CVICALLBACK multimeter_read (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViReal64 value;
   char str[100];
   ViInt32  measurementNumber;
   
   switch (event)
      {
	  case EVENT_COMMIT:
	     hp34401_timeOut (instrumentHandle1, 100000);//IO timeout u ms
		 hp34401_autoZero(instrumentHandle1, VI_TRUE);//podesavanje nule automatsko
		 
		 hp34401_trigSour(instrumentHandle1,2); //Izbor trigera
		 hp34401_sampCoun(instrumentHandle1,1); //Broj semplova=1
		 hp34401_trigCoun(instrumentHandle1,1); //Broj trigera =1
		 hp34401_trigDelAuto(instrumentHandle1, VI_FALSE); //Auto Delay iskljucen
		 hp34401_initImm(instrumentHandle1);//Pocetak merenja, to jest cekanja na triger
		 hp34401_fetc_Q(instrumentHandle1, &value, &measurementNumber); 
		 Fmt (str, "%f", value);
		 MessagePopup(str,str);
 		 break;
	  }
   return 0;
   }

int CVICALLBACK otkucaj (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViReal64 value;
   ViInt32  measurementNumber;
   char  message[256];
   char message_length;
   switch (event)
      {
	  case EVENT_TIMER_TICK:
	     if (process)
		    {
	        
			ag33250a_appl (instrumentHandle2, ag33250a_APPL_SIN, frequency, 1, 0.0);
		    
			hp34401_initImm(instrumentHandle1);//Pocetak merenja, to jest cekanja na triger
			
		    hp34401_fetc_Q(instrumentHandle1, &value, &measurementNumber); 
			
			
			SetCtrlVal (panelHandle, PANEL_FREQ_INPUT, frequency);
			
			value = value*sqrt(2);
			
			measurements[current_point]=20*log(value);
			
			frequencies [current_point]=frequency;
			
			SetCtrlVal (panelHandle, PANEL_AMP_OUT, value);
			
			current_point++;
			
			//DeleteGraphPlot (panelHandle, PANEL_SIGNAL, -1, VAL_IMMEDIATE_DRAW);
			
            PlotXY (panelHandle, PANEL_SIGNAL, frequencies, measurements, current_point,
		    VAL_UNSIGNED_INTEGER, VAL_DOUBLE, VAL_FAT_LINE,
		    VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
			
			
			
			sprintf(message,"%6d %1.2f\n",frequency,value);
			
			WriteFile (file1, message,12);
			
			
			if (current_point!=points) 
			   {
			   frequency+=deltaf;
			   }
			else
			   {
			   process=0;
			   CloseFile(file1);
			   
			   SetCtrlAttribute (panelHandle, PANEL_START, ATTR_DIMMED, 0);
		       SetCtrlAttribute (panelHandle, PANEL_FREQ_START, ATTR_DIMMED, 0);
		       SetCtrlAttribute (panelHandle, PANEL_FREQ_END, ATTR_DIMMED, 0);
		       SetCtrlAttribute (panelHandle, PANEL_POINTS, ATTR_DIMMED, 0);
			}  }
         break;
      }
   return 0;
   }

int CVICALLBACK start (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   unsigned int frequency_start,frequency_end;	
   
   unsigned char message[128];
   
   switch (event)
      {
	  case EVENT_COMMIT:
		 
		 SetCtrlAttribute (panelHandle, PANEL_START, ATTR_DIMMED, 1);
		 
		 SetCtrlAttribute (panelHandle, PANEL_FREQ_START, ATTR_DIMMED, 1);
		 SetCtrlAttribute (panelHandle, PANEL_FREQ_END, ATTR_DIMMED, 1);
		 SetCtrlAttribute (panelHandle, PANEL_POINTS, ATTR_DIMMED, 1);

		 
		 hp34401_confVoltAc(instrumentHandle1); //konfigurisanje da se meri AC
		 hp34401_voltAcRangAuto(instrumentHandle1,VI_TRUE); //ukljucenje auto opsega
		 
		 
		 hp34401_timeOut (instrumentHandle1, 100000);//IO timeout u ms
		 hp34401_autoZero(instrumentHandle1, VI_TRUE);//podesavanje nule automatsko
		 
		 hp34401_trigSour(instrumentHandle1,2); //Izbor trigera
		 
		 hp34401_sampCoun(instrumentHandle1,1); //Broj semplova=1
		 
		 hp34401_trigCoun(instrumentHandle1,1); //Broj trigera =1
		 
		 
		 hp34401_trigDelAuto(instrumentHandle1, VI_FALSE); //Auto Delay iskljucen
		 
		 
		 GetCtrlVal (panelHandle, PANEL_FREQ_START, &frequency_start);
		 GetCtrlVal (panelHandle, PANEL_FREQ_END, &frequency_end);
		 GetCtrlVal (panelHandle, PANEL_POINTS, &points);
		 
		 
		 deltaf = (frequency_end-frequency_start) / (points-1);
		 
		 
		 frequency = frequency_start;
		 
		 
		 current_point=0;
		 
		 FileSelectPopup ("", "*.amp", "*.amp","Ime datoteke", VAL_OK_BUTTON, 0, 1, 1,1, file_name);
		 
		 
		 file1 = OpenFile (file_name, VAL_READ_WRITE, VAL_OPEN_AS_IS, VAL_BINARY);
		 
		 process=1;       
		 
		 
		 break;
      }
   return 0;
   }
